/* Observable (lähettäjä, tarkkailtava) on olio, joka "työntää" (push) dataa vastaanottajalle.
 * Observablea voisi verrata sellaiseen funktioon, joka työntää dataa ulos, 
 * eikä palauta arvoja (Kts. alempana oleva esimerkki).
 * Observable pitää tilata jotta data pääsisi virtaamaan vastaanottajalle. 
 * Olio joita kutsutaan subscriberiksi (tilaaja, tarkkailija) tarkkailee
 * onko dataa tulossa, ja heti kun sitä on tulossa, se vastaanotetaan.   
 *  
 * Observable-Subsciber arkkitehtuurimallissa toimii pub-sub -mekanismi, eli lähettäjä
 * (Observable) julkaisee (pub) datan ja vastaanottaja (Subscriber) tilaa (sub) datan.
 * 
 * Observablea käytetään erityisesti asynkronisen datan kuljettamiseen sovellukseen
 * ja sen välittämiseen sovelluksen sisällä. Observable on siihen kaikkein tehokkain väline.
 * (muita välineitä ovat callback ja promise). Observablen avulla voidaan välittää 
 * streamina tulevaa dataa eli useista peräkkäisistä lähetyksistä koostuvaa datavirtaa.
 * Datavirtaan voidaan kohdistaa operaattoreita joiden avulla sitä voidaan säädellä, kuten
 * suodattaa tai muuntaa dataa toiseen muotoon.
 * 
 * Observablen luomiseen tarvitaan rxjs-kirjasto. Observable ei ole vielä sisäänrakennettuna
 * Javascriptiin, mutta se on todennäköisesti jossain vaiheessa tulossa.
 * 
 * Hyvä esimerkki observablen hyödyistä: https://x-team.com/blog/rxjs-observables/
 */
const Rx = require('rxjs'); // Rx-moduuli sisältää Observable-luokan

// luodaan uusi observable
const observable = new Rx.Observable((subscriber) =>{
   // data lähetetään eli "työnnetään" subscriberille
   subscriber.next('Eka data lähtee observablesta');
   subscriber.next( 'Toka data lähtee observablesta');

});

// Data on jo lähtenyt, koska se on "työnnetty" tulemaan.
// Observablea tarkkaillaan ja jos dataa tulee, se vastaanotetaan subscribe-metodilla.
// Data saadaan subscriben argumenttina olevasta callbackista
observable.subscribe((data)=>{
   console.log(data);    
});

/***************************************************************************/

// Verrataan observablea dataa tuottavaan funktioon:
// Funktio voi lähettää vain yhden tiedon kerrallaan
// Tavallinen funktio ei "työnnä" dataa vastaanottajalle
// vaan data pitää "vetää" funktiosta
function dataProducer(){
   return 'Eka data lähtee funktiosta';
   return 'Toka data lähtee funktiosta'; // rivi jota ei koskaan suoriteta
}

// data ei ole lähtenyt, ennen kuin käyttäjä kutsuu funktiota
const data = dataProducer();
console.log(data);
